/*
 * If this software is used for a game the official „Wurfel Engine“ logo or its name must be visible in an intro screen or main menu.
 * 
 * Copyright 2014 Benedikt Vogler.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * * Neither the name of Benedikt Vogler nor the names of its contributors 
 *   may be used to endorse or promote products derived from this software without specific
 *   prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.BombingGames.WurfelEngine.Core;

import com.BombingGames.WurfelEngine.Core.Gameobjects.AbstractEntity;
import com.BombingGames.WurfelEngine.Core.Gameobjects.AbstractGameObject;
import com.BombingGames.WurfelEngine.Core.Map.AbstractPosition;

/**
 *Saves the information for the rendering. This class is only used in the rendering process and servers as a data tansfer object.
 * @author Benedikt
 */
public class RenderDataDTO {
    private final AbstractPosition pos;
    private final AbstractGameObject object;

     /**
         * Create an Renderobject with a regular Block in the map
         * @param object 
         * @param pos The coordinates where the object should be rendered
         */
    protected RenderDataDTO(final AbstractGameObject object, final AbstractPosition pos) {
        this.pos = pos;
        this.object = object;
    }


    /**
     * Returns the depth of the object.
	 * @param view
     * @return
     */
    public int getDepth(View view) {
        return (int) (pos.getDepth(view)
            + (
            object instanceof AbstractEntity
                ? ((AbstractEntity)object).getDimensionZ()/1.414213562f
                : 0
            )
        );
    }

    /**
     *
     * @return
     */
    public AbstractPosition getPosition() {
        return pos;
    }
    

    /**
     *
     * @return
     */
    public AbstractGameObject getGameObject() {
        return object;
    }

    
}